@@ -10,7 +10,7 @@ require HTML::DOM::Element;
 #require HTML::DOM::NodeList::Magic;
 
 our @ISA = qw'HTML::DOM::Element';
-our $VERSION = '0.040';
+our $VERSION = '0.041';
 
 sub caption {
 	my $old = ((my $self = shift)->content_list)[0];
@@ -348,14 +348,14 @@ L<HTML::DOM::Element::TableCell>
 # ------- HTMLTableCaptionElement interface ---------- #
 
 package HTML::DOM::Element::Caption;
-our $VERSION = '0.040';
+our $VERSION = '0.041';
 our @ISA = 'HTML::DOM::Element';
 *align = \&HTML::DOM::Element::Table::align;
 
 # ------- HTMLTableColElement interface ---------- #
 
 package HTML::DOM::Element::TableColumn;
-our $VERSION = '0.040';
+our $VERSION = '0.041';
 our @ISA = 'HTML::DOM::Element';
 *align = \&HTML::DOM::Element::Table::align;
 sub ch     { shift->_attr('char'   => @_) }
@@ -367,7 +367,7 @@ sub width  { shift->_attr('width'  => @_) }
 # ------- HTMLTableSectionElement interface ---------- #
 
 package HTML::DOM::Element::TableSection;
-our $VERSION = '0.040';
+our $VERSION = '0.041';
 our @ISA = 'HTML::DOM::Element';
 *align  = \&HTML::DOM::Element::Table::align;
 *ch     = \&HTML::DOM::Element::TableColumn::ch;
@@ -420,7 +420,7 @@ sub insertRow {
 # ------- HTMLTableRowElement interface ---------- #
 
 package HTML::DOM::Element::TR;
-our $VERSION = '0.040';
+our $VERSION = '0.041';
 our @ISA = 'HTML::DOM::Element';
 sub rowIndex {
 	my $self = shift;
@@ -506,7 +506,7 @@ sub deleteCell {
 # ------- HTMLTableCellElement interface ---------- #
 
 package HTML::DOM::Element::TableCell;
-our $VERSION = '0.040';
+our $VERSION = '0.041';
 our @ISA = 'HTML::DOM::Element';
 sub cellIndex {
 	my $self = shift;
@@ -528,7 +528,7 @@ sub axis  { shift->_attr('axis'  => @_) }
 sub colSpan  { shift->_attr('colspan'  => @_) }
 sub headers  { shift->_attr('headers'  => @_) }
 sub height   { shift->_attr('height'   => @_) }
-sub noWrap   { shift->_attr('nowrap'   => @_) }
+sub noWrap   { shift->_attr(nowrap => @_ ? $_[0] ? 'nowrap' : undef : ()) }
 sub rowSpan  { shift->_attr('rowspan'  => @_) }
 sub scope    { lc shift->_attr('scope'    => @_) }
 *vAlign  = \&HTML::DOM::Element::TableColumn::vAlign;
